#include <math.h>
#include "complexz.h"
//---------------------------------------------------------------------------
complex::complex(double _rr,double _ii)
{
    re=_rr;
	im=_ii;
}
//---------------------------------------------------------------------------
complex ReIm(double _rr,double _ii)
{
    complex z;
    z.re=_rr;
    z.im=_ii;
	return z;
}
//---------------------------------------------------------------------------
complex Polar(double _rr,double _tt)
{
    complex z;
    z.re=_rr*cos(_tt);
    z.im=_rr*sin(_tt);
	return z;
}
//---------------------------------------------------------------------------
double cabs(complex z)
{
    return sqrt(z.re*z.re+z.im*z.im);
}
//---------------------------------------------------------------------------
double arg(complex z)
{
    return atan2(z.im,z.re);
}
//---------------------------------------------------------------------------
double norm(complex z)
{
    return z.re*z.re+z.im*z.im;
}
//---------------------------------------------------------------------------
double real(complex z)
{
    return z.re;
}
//---------------------------------------------------------------------------
double imag(complex z)
{
    return z.im;
}
//---------------------------------------------------------------------------
complex conj(complex z)
{
    complex c;
    c.re=z.re;
    c.im=-z.im;
    return c;
}
//---------------------------------------------------------------------------
complex operator +(complex a,complex b)
{
    complex z;
    z.re=a.re+b.re;
    z.im=a.im+b.im;
    return z;
}
//---------------------------------------------------------------------------
complex operator +(complex a,double _tt)
{
    complex z;
    z.re=a.re+_tt;
    z.im=a.im;
    return z;
}
//---------------------------------------------------------------------------
complex operator +(double _tt,complex a)
{
    complex z;
    z.re=a.re+_tt;
    z.im=a.im;
    return z;
}
//---------------------------------------------------------------------------
complex operator -(complex a,complex b)
{
    complex z;
    z.re=a.re-b.re;
    z.im=a.im-b.im;
    return z;
}
//---------------------------------------------------------------------------
complex operator -(complex a,double _tt)
{
    complex z;
    z.re=a.re-_tt;
    z.im=a.im;
    return z;
}
//---------------------------------------------------------------------------
complex operator -(double _tt,complex a)
{
    complex z;
    z.re=_tt-a.re;
    z.im=-a.im;
    return z;
}
//---------------------------------------------------------------------------
complex operator *(complex a,complex b)
{
    complex z;
    z.re=a.re*b.re-a.im*b.im;
    z.im=a.re*b.im+a.im*b.re;
    return z;
}
//---------------------------------------------------------------------------
complex operator *(complex a,double _tt)
{
    complex z;
    z.re=a.re*_tt;
    z.im=a.im*_tt;
    return z;
}
//---------------------------------------------------------------------------
complex operator *(double _tt,complex a)
{
    complex z;
    z.re=a.re*_tt;
    z.im=a.im*_tt;
    return z;
}
//---------------------------------------------------------------------------
complex operator /(complex a,complex b)
{
    complex z;
    double nb=1.0/(b.re*b.re+b.im*b.im);
    z.re=(a.re*b.re+a.im*b.im)*nb;
    z.im=(-a.re*b.im+a.im*b.re)*nb;
    return z;
}
//---------------------------------------------------------------------------
complex operator /(complex a,double _tt)
{
    complex z;
    z.re=a.re/_tt;
    z.im=a.im/_tt;
    return z;
}
//---------------------------------------------------------------------------
complex operator /(double _tt,complex a)
{
    complex z;
    double na=1.0/(a.re*a.re+a.im*a.im);
    z.re=_tt*a.re*na;
    z.im=-_tt*a.im*na;
    return z;
}
//---------------------------------------------------------------------------
bool operator ==(complex a,complex b)
{
    if(a.re==b.re&&a.im==b.im) return true;
    else return false;
}
//---------------------------------------------------------------------------
bool operator ==(complex a,double _tt)
{
    if(a.re==_tt&&a.im==0) return true;
    else return false;
}
//---------------------------------------------------------------------------
bool operator ==(double _tt,complex a)
{
    if(a.re==_tt&&a.im==0) return true;
    else return false;
}
//---------------------------------------------------------------------------
bool operator !=(complex a,complex b)
{
    if(a.re==b.re&&a.im==b.im) return false;
    else return true;
}
//---------------------------------------------------------------------------
bool operator !=(complex a,double _tt)
{
    if(a.re==_tt&&a.im==0) return false;
    else return true;
}
//---------------------------------------------------------------------------
bool operator !=(double _tt,complex a)
{
    if(a.re==_tt&&a.im==0) return false;
    else return true;
}
//---------------------------------------------------------------------------
complex complex::operator =(double _tt)
{
    re=_tt;
    im=0;
    return *this;
}
//---------------------------------------------------------------------------
complex complex::operator -()
{
    complex z;
    z.re=-re;
    z.im=-im;
    return z;
}
//---------------------------------------------------------------------------
complex complex::operator +=(complex a)
{
    re+=a.re;
    im+=a.im;
    return *this;
}
//---------------------------------------------------------------------------
complex complex::operator +=(double _tt)
{
    re+=_tt;
    return *this;
}
//---------------------------------------------------------------------------
complex complex::operator -=(complex a)
{
    re-=a.re;
    im-=a.im;
    return *this;
}
//---------------------------------------------------------------------------
complex complex::operator -=(double _tt)
{
    re-=_tt;
    return *this;
}
//---------------------------------------------------------------------------
complex complex::operator *=(complex a)
{
    complex z=*this,y;
    re=z.re*a.re-z.im*a.im;
    im=z.im*a.re+z.re*a.im;
    return *this;
}
//---------------------------------------------------------------------------
complex complex::operator *=(double _tt)
{
    re*=_tt;
    im*=_tt;
    return *this;
}
//---------------------------------------------------------------------------
complex complex::operator /=(complex a)
{
    complex z=*this;
    double na=1.0/(a.re*a.re+a.im*a.im);
    re=(z.re*a.re+z.im*a.im)*na;
    im=(z.im*a.re-z.re*a.im)*na;
    return *this;
}
//---------------------------------------------------------------------------
complex complex::operator /=(double _tt)
{
    re/=_tt;
    im/=_tt;
    return *this;
}
//---------------------------------------------------------------------------
complex exp(complex z)
{
    double er=exp(z.re);
    complex c;
    c.re=er*cos(z.im);
    c.im=er*sin(z.im);
    return c;
}
//---------------------------------------------------------------------------
complex log(complex z)
{
    complex c;
    c.re=log(cabs(z));
    c.im=arg(z);
    return c;
}
//---------------------------------------------------------------------------
complex sqrt(complex z)
{
    double ss=sqrt(cabs(z)),tt=0.5*arg(z);
    complex c;
    c.re=ss*cos(tt);
    c.im=ss*sin(tt);
    return c;
}
//---------------------------------------------------------------------------
complex pow(complex z,double _tt)
{
    double ss=pow(cabs(z),_tt),tt=_tt*arg(z);
    complex c;
    c.re=ss*cos(tt);
    c.im=ss*sin(tt);
    return c;
}
//---------------------------------------------------------------------------
complex pow(complex z,complex y)
{
    double zr=cabs(z),zt=arg(z),yr=real(y),yi=imag(y);
    double ss=pow(zr,yr)*exp(-yi*zt),tt=yi*log(zr)+yr*zt;
    complex c;
    c.re=ss*cos(tt);
    c.im=ss*sin(tt);
    return c;
}
//---------------------------------------------------------------------------
complex pow(double _tt,complex z)
{
    double yr=real(z),yi=imag(z);
    double ss=pow(_tt,yr),tt=yi*log(_tt);
    complex c;
    c.re=ss*cos(tt);
    c.im=ss*sin(tt);
    return c;
}
//---------------------------------------------------------------------------
complex sin(complex z)
{
    double zr=real(z),zi=imag(z);
    complex c;
    c.re=sin(zr)*cosh(zi);
    c.im=cos(zr)*sinh(zi);
    return c;
}
//---------------------------------------------------------------------------
complex sinh(complex z)
{
    double zr=real(z),zi=imag(z);
    complex c;
    c.re=sinh(zr)*cos(zi);
    c.im=cosh(zr)*sin(zi);
    return c;
}
//---------------------------------------------------------------------------
complex cos(complex z)
{
    double zr=real(z),zi=imag(z);
    complex c;
    c.re=cos(zr)*cosh(zi);
    c.im=-sin(zr)*sinh(zi);
    return c;
}
//---------------------------------------------------------------------------
complex cosh(complex z)
{
    double zr=real(z),zi=imag(z);
    complex c;
    c.re=cosh(zr)*cos(zi);
    c.im=sinh(zr)*sin(zi);
    return c;
}
//---------------------------------------------------------------------------
complex tan(complex z)
{
    return sin(z)/cos(z);
}
//---------------------------------------------------------------------------
complex tanh(complex z)
{
    return sinh(z)/cosh(z);
}
//---------------------------------------------------------------------------

